#ifdef Z_SL_OPENGLES

	// --- Uniforms

	uniform highp mat4 u_ModelViewProjection;

	uniform highp mat4 u_Model; 

	uniform lowp sampler2D u_ScreenTexture;

	uniform lowp sampler2D u_Texture;

	uniform lowp sampler2D u_SineTexture;

	uniform highp float u_Offset;

	uniform highp float u_SinOffset;

	uniform highp vec2 u_SinOffsetByPosition;
		

	// --- Attributes
	
	#ifdef Z_SHADER_VERTEX 

		attribute highp vec4 a_Position;

		attribute highp vec2 a_TexCoords;
	
		attribute highp vec2 a_TexCoords2;
		
	#endif
		
	// --- Varyings
	
	varying highp vec2 v_TexCoords2;
	varying highp vec2 v_TexCoords;
	varying highp vec2 v_SinOffset;
		
	// --- Main
	
	#ifdef Z_SHADER_VERTEX

		void main()
		{
			gl_Position = u_ModelViewProjection * a_Position;
			highp vec4 worldPosition = u_Model * a_Position;
			v_SinOffset = vec2(u_SinOffset + dot(u_SinOffsetByPosition, worldPosition.xy), 0.0);
			v_TexCoords2 = a_TexCoords2;
			v_TexCoords = a_TexCoords;
		}
		
	#else

		void main()
		{
			lowp float texValue = texture2D(u_Texture, v_TexCoords).r;

			highp vec2 screenTexCoords = v_TexCoords2;

			lowp float sineTexValue = texture2D(u_SineTexture, v_SinOffset).a;
            // lowp float sineValue = (2.0 * sineTexValue - 1.0); // 2.0 overflows lowp.
            lowp float sineValue = (sineTexValue - 1.0) + sineTexValue;
			screenTexCoords.x += texValue * u_Offset * sineValue;

			lowp vec3 screenTexValue = texture2D(u_ScreenTexture, screenTexCoords).rgb;

			gl_FragColor = vec4(screenTexValue, 1.0);

			//gl_FragColor = vec4(sineTexValue, sineTexValue, sineTexValue, 1.0);
		}
		
	#endif
		
#endif